from yafowil.base import factory


def bs_field_class(widget, data):
    if data.errors:
        return 'form-group has-error'
    return 'form-group'


def configure_factory():
    # set theme
    factory.theme = 'bootstrap3'

    # common defaults
    factory.defaults['text.class'] = 'text form-control'

    factory.defaults['textarea.class'] = 'textarea form-control'

    factory.defaults['lines.class'] = 'lines form-control'

    factory.defaults['password.class'] = 'password form-control'

    factory.defaults['select.class'] = 'select'
    factory.defaults['select.block_class'] = 'form-control'
    factory.defaults['select.radio_wrapper_class'] = 'radio'
    factory.defaults['select.checkbox_wrapper_class'] = 'checkbox'

    factory.defaults['submit.class'] = 'btn btn-default'

    factory.defaults['email.class'] = 'email form-control'

    factory.defaults['url.class'] = 'url form-control'

    factory.defaults['search.class'] = 'search form-control'

    factory.defaults['number.class'] = 'number form-control'

    factory.defaults['label.class'] = 'control-label'

    factory.defaults['field.class'] = bs_field_class

    factory.defaults['error.position'] = 'after'
    factory.defaults['error.tag'] = 'span'
    factory.defaults['error.class'] = 'help-block'
    factory.defaults['error.message_class'] = 'text-danger'

    factory.defaults['help.position'] = 'after'
    factory.defaults['help.tag'] = 'p'
    factory.defaults['help.class'] = 'help-block'

    # yafowil.widget.array
    factory.defaults['array.table_class'] = 'table table-condensed'

    # yafowil.widget.autocomplete
    factory.defaults['autocomplete.class'] = 'autocomplete form-control'

    # yafowil.widget.chosen
    factory.defaults['chosen.class'] = 'chosen form-control'

    # yafowil.widget.datetime
    factory.defaults['datetime.datepicker_class'] = 'datepicker form-control'
    factory.defaults['datetime.timepicker_class'] = 'timepicker form-control'
    factory.defaults['time.timepicker_class'] = 'timepicker form-control'

    # yafowil.widget.dict
    factory.defaults['dict.table_class'] = 'dictwidget table table-condensed'
    factory.defaults['dict.key_class'] = 'form-control'
    factory.defaults['dict.value_class'] = 'form-control'

    # yafowil.widget.wysihtml5
    factory.defaults['wysihtml5.class'] = 'wysihtml5 form-control'


def register_macros():
    # common
    factory.register_macro('form', 'form', {'form.class': 'form-horizontal'})
    factory.register_macro(
        'field',
        'field:label:div:help:error',
        {
            'label.class_add': 'col-sm-2',
            'div.class_add': 'col-sm-10'
        }
    )
    factory.register_macro(
        'button',
        'submit', {
            'submit.class': 'btn',
            'submit.class_add': 'btn-default'
        }
    )

    # yafowil.widget.array
    factory.register_macro(
        'array',
        'field:label:help:error:array',
        {
            'array.label': ' ',
            'field.class': bs_field_class,
            'label.class_add': 'col-sm-2',
            'array.class_add': 'col-sm-10',
            'help.class_add': 'col-sm-offset-2 col-sm-10',
            'error.class_add': 'col-sm-offset-2 col-sm-10'
        }
    )
    factory.register_macro('arrayfield', 'field:label:help:error', {})



from yafowil.base import factory
from yafowil.utils import attr_value


def configure_factory():
    # set theme
    factory.theme = 'bootstrap5'

    # wrapper div for one input
    factory.defaults['field.class'] = 'field'

    # label marker
    factory.defaults['label.class'] = 'form-label'

    # label required marker (plone)
    factory.defaults['label.required_class'] = 'required'

    # common defaults
    # form-control:
    # errors: https://v5.getbootstrap.com/docs/5.0/forms/overview/
    bs5_input_blueprints = [
        'text',
        'textarea',
        'lines',
        'password',
        'email',
        'url',
        'search',
        'number',
        'file',
    ]
    for blueprint_name in bs5_input_blueprints:
        factory.defaults['{0}.class'.format(blueprint_name)] = 'form-control'
        factory.defaults['{0}.error_class'.format(blueprint_name)] = 'is-invalid'
        factory.defaults['{0}.valid_class'.format(blueprint_name)] = 'is-valid'


    factory.defaults['submit.class'] = 'btn btn-primary'
    factory.defaults['button.class'] = 'btn btn-primary'

    factory.defaults['error.position'] = 'after'
    factory.defaults['error.tag'] = 'div'
    factory.defaults['error.class'] = 'invalid-feedback'
    factory.defaults['error.message_class'] = None
    factory.defaults['error.message_tag'] = None

    factory.defaults['help.position'] = 'after'
    factory.defaults['help.tag'] = 'div'
    factory.defaults['help.class'] = 'form-text'

    # file
    factory.defaults['file.class'] = 'form-control'

    # select
    factory.defaults['select.error_class'] = 'is-invalid'
    factory.defaults['select.valid_class'] = 'is-valid'
    factory.defaults['select.block_class'] = 'form-control'
    factory.defaults['select.radio_input_class'] = 'form-check-input'
    factory.defaults['select.radio_label_class'] = 'form-check-label'
    factory.defaults['select.radio_wrapper_class'] = 'form-check'
    factory.defaults['select.checkbox_input_class'] = 'form-check-input'
    factory.defaults['select.checkbox_label_class'] = 'form-check-label'
    factory.defaults['select.checkbox_wrapper_class'] = 'form-check'
    factory.defaults['select.listing_label_position'] = 'after'

    # single checkbox
    factory.defaults['checkbox.error_class'] = 'is-invalid'
    factory.defaults['checkbox.valid_class'] = 'is-valid'
    factory.defaults['checkbox.class'] = 'form-check-input'

    # yafowil.widget.array
    factory.defaults['array.table_class'] = 'table table-condensed'

    # yafowil.widget.autocomplete
    factory.defaults['autocomplete.class'] = 'autocomplete form-control'

    # yafowil.widget.chosen
    factory.defaults['chosen.class'] = 'chosen form-control'

    # yafowil.widget.datetime
    factory.defaults['datetime.datepicker_class'] = 'datepicker form-control'
    factory.defaults['datetime.timepicker_class'] = 'timepicker form-control'
    factory.defaults['time.timepicker_class'] = 'timepicker form-control'

    # yafowil.widget.dict
    factory.defaults['dict.table_class'] = 'dictwidget table table-condensed'
    factory.defaults['dict.key_class'] = 'form-control'
    factory.defaults['dict.value_class'] = 'form-control'

    # yafowil.widget.wysihtml5
    factory.defaults['wysihtml5.class'] = 'wysihtml5 form-control'


def register_macros():
    # common
    factory.register_macro('form', 'form', {})
    factory.register_macro('field', 'field:label:help:error', {})
    factory.register_macro(
        'button',
        'button',
        {
            'button.class': 'btn',
            'button.class_add': 'btn-primary'
        }
    )

    # yafowil.widget.array
    factory.register_macro(
        'array',
        'field:label:help:error:array',
        {
            'array.label': ' ',
            'label.class_add': 'col-sm-2',
            'array.class_add': 'col-sm-10',
            'help.class_add': 'col-sm-offset-2 col-sm-10',
            'error.class_add': 'col-sm-offset-2 col-sm-10'
        }
    )
    factory.register_macro('arrayfield', 'field:label:help:error', {})



from node.tests import NodeTestCase
from yafowil.base import factory
import os
import unittest
import webresource as wr
import yafowil.loader  # noqa


def add_jquery_js(resources):
    resources.add(wr.ScriptResource(
        name='jquery-js',
        path='jquery',
        resource='jquery.js'
    ))


def np(path):
    return path.replace('/', os.path.sep)


class TestBootstrap(NodeTestCase):

    def test_bs3_resources(self):
        factory.theme = 'bootstrap'
        resources = factory.get_resources('yafowil.bootstrap', False)
        factory.theme = 'bootstrap3'
        self.assertTrue(
            factory.get_resources('yafowil.bootstrap', False) is resources
        )

        resources = factory.get_resources('yafowil.bootstrap')
        self.assertTrue(resources.directory.endswith(np('/resources/bs3')))
        self.assertEqual(resources.name, 'yafowil.bootstrap')
        self.assertEqual(resources.path, 'bootstrap')

        scripts = resources.scripts
        self.assertEqual(len(scripts), 1)

        self.assertTrue(scripts[0].directory.endswith(np('/bs3/js')))
        self.assertEqual(scripts[0].path, 'bootstrap/js')
        self.assertEqual(scripts[0].file_name, 'bootstrap.min.js')
        self.assertTrue(os.path.exists(scripts[0].file_path))

        styles = resources.styles
        self.assertEqual(len(styles), 2)

        self.assertTrue(styles[0].directory.endswith(np('/bs3/css')))
        self.assertEqual(styles[0].path, 'bootstrap/css')
        self.assertEqual(styles[0].file_name, 'bootstrap.min.css')
        self.assertTrue(os.path.exists(styles[0].file_path))

        self.assertTrue(styles[1].directory.endswith(np('/bs3/css')))
        self.assertEqual(styles[1].path, 'bootstrap/css')
        self.assertEqual(styles[1].file_name, 'bootstrap-theme.min.css')
        self.assertTrue(os.path.exists(styles[1].file_path))

        add_jquery_js(resources)
        resolver = wr.ResourceResolver(resources)
        renderer = wr.ResourceRenderer(resolver, base_url='')
        self.checkOutput("""
        <link href="/bootstrap/css/bootstrap.min.css" media="all"
              rel="stylesheet" type="text/css" />
        <link href="/bootstrap/css/bootstrap-theme.min.css" media="all"
              rel="stylesheet" type="text/css" />
        <script src="/jquery/jquery.js"></script>
        <script src="/bootstrap/js/bootstrap.min.js"></script>
        """, renderer.render())

    def test_bs4_resources(self):
        factory.theme = 'bootstrap4'
        resources = factory.get_resources('yafowil.bootstrap')
        self.assertEqual(resources.name, 'yafowil.bootstrap')
        self.assertTrue(resources.directory.endswith(np('/resources/bs4')))
        self.assertEqual(resources.path, 'bootstrap')

        scripts = resources.scripts
        self.assertEqual(len(scripts), 1)

        self.assertTrue(scripts[0].directory.endswith(np('/bs4/js')))
        self.assertEqual(scripts[0].path, 'bootstrap/js')
        self.assertEqual(scripts[0].file_name, 'bootstrap.min.js')
        self.assertTrue(os.path.exists(scripts[0].file_path))

        styles = resources.styles
        self.assertEqual(len(styles), 1)

        self.assertTrue(styles[0].directory.endswith(np('/bs4/css')))
        self.assertEqual(styles[0].path, 'bootstrap/css')
        self.assertEqual(styles[0].file_name, 'bootstrap.min.css')
        self.assertTrue(os.path.exists(styles[0].file_path))

        add_jquery_js(resources)
        resolver = wr.ResourceResolver(resources)
        renderer = wr.ResourceRenderer(resolver, base_url='')
        self.checkOutput("""
        <link href="/bootstrap/css/bootstrap.min.css" media="all"
              rel="stylesheet" type="text/css" />
        <script src="/jquery/jquery.js"></script>
        <script src="/bootstrap/js/bootstrap.min.js"></script>
        """, renderer.render())

    def test_bs5_resources(self):
        factory.theme = 'bootstrap5'
        resources = factory.get_resources('yafowil.bootstrap')
        self.assertEqual(resources.name, 'yafowil.bootstrap')
        self.assertTrue(resources.directory.endswith(np('/resources/bs5')))
        self.assertEqual(resources.path, 'bootstrap')

        scripts = resources.scripts
        self.assertEqual(len(scripts), 1)

        self.assertTrue(scripts[0].directory.endswith(np('/bs5/js')))
        self.assertEqual(scripts[0].path, 'bootstrap/js')
        self.assertEqual(scripts[0].file_name, 'bootstrap.min.js')
        self.assertTrue(os.path.exists(scripts[0].file_path))

        styles = resources.styles
        self.assertEqual(len(styles), 1)

        self.assertTrue(styles[0].directory.endswith(np('/bs5/css')))
        self.assertEqual(styles[0].path, 'bootstrap/css')
        self.assertEqual(styles[0].file_name, 'bootstrap.min.css')
        self.assertTrue(os.path.exists(styles[0].file_path))

        resolver = wr.ResourceResolver(resources)
        renderer = wr.ResourceRenderer(resolver, base_url='')
        self.checkOutput("""
        <script src="/bootstrap/js/bootstrap.min.js"></script>
        <link href="/bootstrap/css/bootstrap.min.css" media="all"
              rel="stylesheet" type="text/css" />
        """, renderer.render())

if __name__ == '__main__':
    unittest.main()



from yafowil.base import factory
from yafowil.bootstrap import bs3
from yafowil.bootstrap import bs4
from yafowil.bootstrap import bs5
from yafowil.utils import entry_point
import os
import warnings
import webresource as wr


resources_dir = os.path.join(os.path.dirname(__file__), 'resources')


##############################################################################
# Bootstrap 3
##############################################################################

# webresource ################################################################

bs3_resources_dir = os.path.join(resources_dir, 'bs3')
bs3_scripts_dir = os.path.join(bs3_resources_dir, 'js')
bs3_styles_dir = os.path.join(bs3_resources_dir, 'css')

bs3_resources = wr.ResourceGroup(
    name='yafowil.bootstrap',
    directory=bs3_resources_dir,
    path='bootstrap'
)
bs3_resources.add(wr.ScriptResource(
    name='bootstrap-js',
    depends='jquery-js',
    directory=bs3_scripts_dir,
    path='bootstrap/js',
    resource='bootstrap.js',
    compressed='bootstrap.min.js'
))
bs3_resources.add(wr.StyleResource(
    name='bootstrap-css',
    directory=bs3_styles_dir,
    path='bootstrap/css',
    resource='bootstrap.css',
    compressed='bootstrap.min.css'
))
bs3_resources.add(wr.StyleResource(
    name='bootstrap-theme-css',
    depends='bootstrap-css',
    directory=bs3_styles_dir,
    path='bootstrap/css',
    resource='bootstrap-theme.css',
    compressed='bootstrap-theme.min.css'
))

# B/C resources ##############################################################

js_bs3 = [{
    'group': 'bootstrap.dependencies',
    'resource': 'js/bootstrap.min.js',
    'order': 20
}]
css_bs3 = [{
    'group': 'bootstrap.dependencies',
    'resource': 'css/bootstrap.min.css',
    'order': 10
}, {
    'group': 'bootstrap.dependencies',
    'resource': 'css/bootstrap-theme.min.css',
    'order': 11,
}]


##############################################################################
# Bootstrap 4
##############################################################################

# webresource ################################################################

bs4_resources_dir = os.path.join(resources_dir, 'bs4')
bs4_scripts_dir = os.path.join(bs4_resources_dir, 'js')
bs4_styles_dir = os.path.join(bs4_resources_dir, 'css')

bs4_resources = wr.ResourceGroup(
    name='yafowil.bootstrap',
    directory=bs4_resources_dir,
    path='bootstrap',
)
bs4_resources.add(wr.ScriptResource(
    name='bootstrap-js',
    depends='jquery-js',
    directory=bs4_scripts_dir,
    path='bootstrap/js',
    resource='bootstrap.js',
    compressed='bootstrap.min.js'
))
bs4_resources.add(wr.StyleResource(
    name='bootstrap-css',
    directory=bs4_styles_dir,
    path='bootstrap/css',
    resource='bootstrap.css',
    compressed='bootstrap.min.css'
))

# B/C resources ##############################################################

js_bs4 = [{
    'group': 'bootstrap.dependencies',
    'resource': 'js/bootstrap.min.js',
    'order': 20
}]
css_bs4 = [{
    'group': 'bootstrap.dependencies',
    'resource': 'css/bootstrap.min.css',
    'order': 10
}]


##############################################################################
# Bootstrap 5
##############################################################################

# webresource ################################################################

bs5_resources_dir = os.path.join(resources_dir, 'bs5')
bs5_scripts_dir = os.path.join(bs5_resources_dir, 'js')
bs5_styles_dir = os.path.join(bs5_resources_dir, 'css')

bs5_resources = wr.ResourceGroup(
    name='yafowil.bootstrap',
    directory=bs5_resources_dir,
    path='bootstrap'
)
bs5_resources.add(wr.ScriptResource(
    name='bootstrap-js',
    directory=bs5_scripts_dir,
    path='bootstrap/js',
    resource='bootstrap.js',
    compressed='bootstrap.min.js'
))
bs5_resources.add(wr.StyleResource(
    name='bootstrap-css',
    directory=bs5_styles_dir,
    path='bootstrap/css',
    resource='bootstrap.css',
    compressed='bootstrap.min.css'
))

# B/C resources ##############################################################

js_bs5 = [{
    'group': 'bootstrap.dependencies',
    'resource': 'js/bootstrap.min.js',
    'order': 20
}]
css_bs5 = [{
    'group': 'bootstrap.dependencies',
    'resource': 'css/bootstrap.min.css',
    'order': 10
}]


##############################################################################
# Registration
##############################################################################

@entry_point(order=20)
def register():
    widget_name = 'yafowil.bootstrap'

    # Bootstrap 3
    # bootstrap is legacy name for bootstrap3
    factory.register_theme(
        ['bootstrap', 'bootstrap3'],
        widget_name,
        bs3_resources_dir,
        js=js_bs3,
        css=css_bs3
    )
    factory.register_resources(
        ['bootstrap', 'bootstrap3'],
        widget_name,
        bs3_resources
    )

    # Bootstrap 4
    factory.register_theme(
        'bootstrap4',
        widget_name,
        bs4_resources_dir,
        js=js_bs4,
        css=css_bs4
    )
    factory.register_resources('bootstrap4', widget_name, bs4_resources)

    # Bootstrap 5
    factory.register_theme(
        'bootstrap5',
        widget_name,
        bs5_resources_dir,
        js=js_bs5,
        css=css_bs5
    )
    factory.register_resources('bootstrap5', widget_name, bs5_resources)


##############################################################################
# Configuration
##############################################################################

def configure_factory(theme):
    """Configure the Yafowil factory with theme specific macros and defaults.
    """
    if theme == 'bootstrap':
        warnings.warn(
            'Legacy ``bootstrap`` theme is deprecated. Please use '
            '``bootstrap3`` instead.'
        )
        baseline = bs3
    elif theme == 'bootstrap3':
        baseline = bs3
    elif theme == 'bootstrap4':
        baseline = bs4
    elif theme == 'bootstrap5':
        baseline = bs5
    else:
        raise ValueError('Available themes: bootstrap3, bootstrap4, bootstrap5')
    baseline.configure_factory()
    baseline.register_macros()
